function [means,stds,rejrates,PR] = CD_sizepower_PMF(rho,loadingtype,errortype)
% The function mainly obtains rejection means, stds, and rejection
% rates for CD0, CD, CD*, CDw, CDw+,CDz+. Here rho defines the cross sectional
% dependence in error term, alpha defines strength of factors while
% errortype defines the error type.
ens = [50 100 200 500 1000];
ties = ens;    
numsim = 2000;

CD0           = zeros(numsim,1);
PR0           = zeros(numsim,1);

CD2           = zeros(numsim,1);
CDs2          = zeros(numsim,1);
CDw2          = zeros(numsim,1);
CDwn2         = zeros(numsim,1);
CDzn2         = zeros(numsim,1);
PR2           = zeros(numsim,1);

CD3           = zeros(numsim,1);
CDs3          = zeros(numsim,1);
CDw3          = zeros(numsim,1);
CDwn3         = zeros(numsim,1);
CDzn3         = zeros(numsim,1);
PR3           = zeros(numsim,1);

CD4           = zeros(numsim,1);
CDs4          = zeros(numsim,1);
CDw4          = zeros(numsim,1);
CDwn4         = zeros(numsim,1);
CDzn4         = zeros(numsim,1);
PR4           = zeros(numsim,1);           

allCD0           = zeros(numsim,0);
allPR0           = zeros(numsim,0);

allCD2           = zeros(numsim,0);
allCDs2          = zeros(numsim,0);
allCDw2          = zeros(numsim,0);
allCDwn2         = zeros(numsim,0);
allCDzn2         = zeros(numsim,0);

allPR2           = zeros(numsim,0);

allCD3           = zeros(numsim,0);
allCDs3          = zeros(numsim,0);
allCDw3          = zeros(numsim,0);
allCDwn3         = zeros(numsim,0);
allCDzn3         = zeros(numsim,0);
allPR3           = zeros(numsim,0);

allCD4           = zeros(numsim,0);
allCDs4          = zeros(numsim,0);
allCDw4          = zeros(numsim,0);
allCDwn4         = zeros(numsim,0);
allCDzn4         = zeros(numsim,0);
allPR4           = zeros(numsim,0);

rng(27072021,'combRecursive');

%Set seed on workers
if isempty(gcp('nocreate')) == 1
parpool;
end
spmd
    rng(27072021,'combRecursive');
end


for n = ens
    for t = ties
    status = ['Simulating N=',num2str(n),', T=',num2str(t)];
    display(status);
    parfor itr = 1:numsim
    % This should ensure reproducable random numbers
        stream = RandStream.getGlobalStream();
        stream.Substream = itr+1;                  
        %%% Do DGP
        
        % factor loading
        switch loadingtype 
            case 1 
                alpha1 = 1;
                alpha2 = 1;
                
            case 2
                alpha1 = 2/3;%1,2/3
                alpha2 = 1;%1,2/3
            case 3
                alpha1 = 2/3; %2/3,1/2
                alpha2 = 1/2; %2/3,1/2
            case 4
                alpha1 = 1/2;
                alpha2 = 1/2;
        end
        numsfactor1 = floor(n^alpha1);
        g_s1        = 0.5+sqrt(0.5)*randn(numsfactor1,1);
        g_y1        = zeros(n,1);
        index1      = randperm(n);
        g_y1(index1(1:numsfactor1)) = g_s1; 
        
        numsfactor2 = floor(n^alpha2);
        g_s2        = 1+1*randn(numsfactor2,1); %(0.5,0.5), (1,1)
        g_y2        = zeros(n,1);
        index2      = randperm(n);
        g_y2(index2(1:numsfactor2)) = g_s2;
        
	
        % factor
        rhof = 0.9;
        R = [rhof,0;0,rhof];
        xi1 = (chi2rnd(2,t+50,1)-2)/2;
        xi2 = (chi2rnd(2,t+50,1)-2)/2;
        mp   = 1-rhof.^2;
        v1 = sqrt(mp)*xi1;
	    v2 = sqrt(mp)*xi2;
        f0 = zeros(2,t+50);
        f0(:,1) = [v1(1);v2(1)];
        for s = 2:(t+50)
            f0(:,s) = R*f0(:,s-1)+[v1(s);v2(s)];
        end
        f = f0(:,51:t+50);
        
        % error
        sig2_y = 0.5+0.5*chi2rnd(2,1,n);
        rootsig2_y = sig2_y.^(1/2);  
        
        % factor loading
        l_y1  = g_y1.*rootsig2_y';
        l_y2  = g_y2.*rootsig2_y';
              
        switch errortype
            case 1
                xi = randn(n,t);
            case 2
                xi = (chi2rnd(2,n,t)-2)/2;
            case 3
             %   xi = trnd(4,n,t)/sqrt(2); %3,4,6,8,12
		xi = randn(n,t);
                xi = sqrt((6-2)./chi2rnd(6,1,t)).*xi;
        end
        W0 = zeros(n,n);
        W0(1,1:3) = 1;
        W0(2,1:4) = 1;
        W0(n-1,n-3:n) = 1;
        W0(n,n-2:n)   = 1;
        for i = 3:n-2
            W0(i,i-2:i+2) = 1;
        end
        % standardize the weight matrix
        W = W0./sum(W0,2);
        Winv = pinv(eye(n)-rho*W);% derive the inverse of IN-rho*W
        c    = sqrt(n/trace(Winv*Winv'));
        e_y  = c*Winv*xi;   % n x t
        e_y  = e_y';   %t x n
        
            
        % Generate data
        y       = zeros(t,n);
        y0      = zeros(t,n);
        a       = 1+sqrt(2)*randn(n,1);
        for i = 1:n  
            y0(:,i)  = 1/sqrt(2)*(f(1,:)'*l_y1(i)+f(2,:)'*l_y2(i)) + rootsig2_y(i)*e_y(:,i);
            y(:,i)   = a(i) + y0(:,i);   
        end
        
        % compute CD0 by assuming f is known
        xx0    = [ones(t,1),f'];
        beta0  = pinv(xx0'*xx0)*xx0'*y;
        res0   = y-xx0*beta0;
        PR0(itr) = 1-res0(:)'*res0(:)/sum(sum((y-mean(y)).^2)); 
            
        rhohats0   = corrcoef(res0); % N x N           
        allrho0    = triu(rhohats0,1); 
        res_cov0   = t^(-1)*(res0'*res0); % N x N
        res_var0   = diag(res_cov0); % N x 1              

        CDscaling   = sqrt(2*t/(n*(n-1)));
        CDtemp0     = CDscaling*sum(allrho0(:));
        CD0(itr)    = CDtemp0;
            
        % standardize and PCA
        ys      = (y-mean(y))./std(y);
        [PC1,PC2] = eig(ys*ys');
        
        
        % m=2
        f       = PC1(:,end-1:end);
        xx      = [ones(t,1),f];
        
        beta20 = pinv(xx'*xx)*xx'*y;
        res20  = y-xx*beta20;
        PR2(itr)     = 1-res20(:)'*res20(:)/sum(sum((y-mean(y)).^2));
        
        beta    = pinv(xx'*xx)*xx'*ys;
        res2       = ys-xx*beta;
        betasum = diag(sqrt(beta(2:end,:)*beta(2:end,:)'/n));
        gmma2    = pinv(diag(betasum,0))*beta(2:end,:);
        
        out2 = CD_generation(res2,n,t,gmma2);
        CD2(itr) = out2.CD;
        CDs2(itr) = out2.CDs;
        CDw2(itr) = out2.CDw;
        CDwn2(itr) = out2.CDwn;
        CDzn2(itr) = out2.CDzn; 
        
       
        % m=3
        f       = PC1(:,end-2:end);
        xx      = [ones(t,1),f];
        
        beta30 = pinv(xx'*xx)*xx'*y;
        res30  = y-xx*beta30;
        PR3(itr)     = 1-res30(:)'*res30(:)/sum(sum((y-mean(y)).^2));
        
        beta    = pinv(xx'*xx)*xx'*ys;
        res3       = ys-xx*beta;
        betasum = diag(sqrt(beta(2:end,:)*beta(2:end,:)'/n));
        gmma3    = pinv(diag(betasum,0))*beta(2:end,:);
        
        out3 = CD_generation(res3,n,t,gmma3);
        CD3(itr) = out3.CD;
        CDs3(itr) = out3.CDs;
        CDw3(itr) = out3.CDw;
        CDwn3(itr) = out3.CDwn;
        CDzn3(itr) = out3.CDzn;
        
        % m=4
        f       = PC1(:,end-3:end);
        xx      = [ones(t,1),f];
        
        beta40 = pinv(xx'*xx)*xx'*y;
        res40  = y-xx*beta40;
        PR4(itr)     = 1-res40(:)'*res40(:)/sum(sum((y-mean(y)).^2));
        
        beta    = pinv(xx'*xx)*xx'*ys;
        res4       = ys-xx*beta;
        betasum = diag(sqrt(beta(2:end,:)*beta(2:end,:)'/n));
        gmma4    = pinv(diag(betasum,0))*beta(2:end,:);
        
        out4 = CD_generation(res4,n,t,gmma4);
        CD4(itr) = out4.CD;
        CDs4(itr) = out4.CDs;
        CDw4(itr) = out4.CDw;
        CDwn4(itr) = out4.CDwn;
        CDzn4(itr) = out4.CDzn;
        
        % calculate the PR^2
        % assume we know the true number of factors    
    end
    allCD0    = cat(2,allCD0, CD0);
    allPR0    = cat(2,allPR0,PR0);
    allCD2     = cat(2,allCD2,CD2);
    allCDs2    = cat(2,allCDs2,CDs2);
    allCDw2    = cat(2,allCDw2,CDw2);
    allCDwn2   = cat(2,allCDwn2,CDwn2);
    allCDzn2   = cat(2,allCDzn2,CDzn2);
     allPR2    = cat(2,allPR2,PR2);
    
    allCD3     = cat(2,allCD3,CD3);
    allCDs3    = cat(2,allCDs3,CDs3);
    allCDw3    = cat(2,allCDw3,CDw3);
    allCDwn3   = cat(2,allCDwn3,CDwn3);
    allCDzn3   = cat(2,allCDzn3,CDzn3);
     allPR3    = cat(2,allPR3,PR3);
    
    allCD4     = cat(2,allCD4,CD4);
    allCDs4    = cat(2,allCDs4,CDs4);
    allCDw4    = cat(2,allCDw4,CDw4);
    allCDwn4   = cat(2,allCDwn4,CDwn4);
    allCDzn4   = cat(2,allCDzn4,CDzn4);
     allPR4    = cat(2,allPR4,PR4);
    end
end
means    = [mean(allCD0,1);mean(allCD2,1);mean(allCDs2,1);mean(allCDw2,1);...
    mean(allCDwn2,1);mean(allCDzn2,1);...
    mean(allCD3,1);mean(allCDs3,1);mean(allCDw3,1);
    mean(allCDwn3,1);mean(allCDzn3,1);...
    mean(allCD4,1);mean(allCDs4,1);mean(allCDw4,1);
    mean(allCDwn4,1);mean(allCDzn4,1);];
stds     = [std(allCD0,1);std(allCD2,1);std(allCDs2,1);std(allCDw2,1);...
    std(allCDwn2,1);std(allCDzn2,1);...
    std(allCD3,1);std(allCDs3,1);std(allCDw3,1);...
    std(allCDwn3,1);std(allCDzn3,1);...
    std(allCD4,1);std(allCDs4,1);std(allCDw4,1);...
    std(allCDwn4,1);std(allCDzn4,1)];
rejrates = [mean(abs(allCD0) > 1.96);mean(abs(allCD2) > 1.96);mean(abs(allCDs2) > 1.96);...
    mean(abs(allCDw2) > 1.96);mean(abs(allCDwn2) > 1.96);mean(abs(allCDzn2) > 1.96);...
    mean(abs(allCD3) > 1.96);mean(abs(allCDs3) > 1.96);...
    mean(abs(allCDw3) > 1.96);mean(abs(allCDwn3) > 1.96);mean(abs(allCDzn3) > 1.96);...
    mean(abs(allCD4) > 1.96);mean(abs(allCDs4) > 1.96);...
    mean(abs(allCDw4) > 1.96);mean(abs(allCDwn4) > 1.96);mean(abs(allCDzn4) > 1.96)
    ];
rejrates = round(100*rejrates,1);
PR       = [mean(allPR0);mean(allPR2);mean(allPR3);mean(allPR4)];

% arrange the data to the required table form
means    = reshaper(means,80,5);
stds     = reshaper(stds,80,5);
rejrates = reshaper(rejrates,80,5);
PR       = reshaper(PR,20,5);
end
function a=reshaper(A,a,b)
%reshape
a=reshape(A.',b,a).';
end
