function [means,stds,rejrates,PR] = CD_sizepower_XPSF(rho,loadingtype,errortype)
% The function mainly obtains rejection means, stds, and rejection
% rates for CD0, CD, CD*, CDw, CDw+,CDz+. Here rho defines the cross sectional
% dependence in error term, alpha defines strength of factors while
% errortype defines the error type.
ens = [50 100 200 500 1000];
ties = ens;    
numsim = 2000;

CD0           = zeros(numsim,1);

PR0           = zeros(numsim,1);

CD1           = zeros(numsim,1);
CDs1          = zeros(numsim,1);
CDw1          = zeros(numsim,1);
CDwn1         = zeros(numsim,1);
CDzn1         = zeros(numsim,1);

PR1           = zeros(numsim,1);

CD2           = zeros(numsim,1);
CDs2          = zeros(numsim,1);
CDw2          = zeros(numsim,1);
CDwn2         = zeros(numsim,1);
CDzn2         = zeros(numsim,1);

PR2           = zeros(numsim,1);  

allCD0           = zeros(numsim,0);
allPR0           = zeros(numsim,0);

allCD1           = zeros(numsim,0);
allCDs1          = zeros(numsim,0);
allCDw1          = zeros(numsim,0);
allCDwn1         = zeros(numsim,0);
allCDzn1         = zeros(numsim,0);
allPR1           = zeros(numsim,0);

allCD2           = zeros(numsim,0);
allCDs2          = zeros(numsim,0);
allCDw2          = zeros(numsim,0);
allCDwn2         = zeros(numsim,0);
allCDzn2         = zeros(numsim,0);

allPR2           = zeros(numsim,0);

rng(27072021,'combRecursive');

%Set seed on workers
if isempty(gcp('nocreate')) == 1
parpool;
end
spmd
    rng(27072021,'combRecursive');
end


for n = ens
    for t = ties
    status = ['Simulating N=',num2str(n),', T=',num2str(t)];
    display(status);
    parfor itr = 1:numsim
    % This should ensure reproducable random numbers
        stream = RandStream.getGlobalStream();
        stream.Substream = itr+1;                  
        %%% Do DGP
        rhog = 0.8;        
        vg0 = randn(50+t,1);
        vg = sqrt(1-rhog.^2)*vg0;
        g0 = zeros(1,t+50);
        g0(:,1) = vg(1);
        for s = 2:(t+50)
            g0(s) = rhog*g0(s-1)+vg(s);
        end
        g = g0(51:t+50);
        
        % factor loading
        switch loadingtype 
            case 1 
                alpha1 = 1;    
            case 2
                alpha1 = 2/3;
            case 3
                alpha1 = 1/2;
        end
        numsfactor1 = floor(n^alpha1);
        g_s1        = 0.5+sqrt(0.5)*randn(numsfactor1,1);
        g_y1        = zeros(n,1);
        index1      = randperm(n);
        g_y1(index1(1:numsfactor1)) = g_s1; 
        
	
        % factor
        rhof = 0.9;
        R = [rhof,0;0,rhof];
        xi1 = (chi2rnd(2,t+50,1)-2)/2;
        xi2 = (chi2rnd(2,t+50,1)-2)/2;
        mp   = 1-rhof.^2;
        v1 = sqrt(mp)*xi1;
	    v2 = sqrt(mp)*xi2;
        f0 = zeros(2,t+50);
        f0(:,1) = [v1(1);v2(1)];
        for s = 2:(t+50)
            f0(:,s) = R*f0(:,s-1)+[v1(s);v2(s)];
        end
        f = f0(:,51:t+50);

      
        % error
        sig2_y = 0.5+0.5*chi2rnd(2,1,n);
        rootsig2_y = sig2_y.^(1/2);         
        
        % regressors
        rhox = 0.95*rand(1,n);
        vx0 = randn(50+t,n);
        vx = vx0.*sqrt(1-rhox.^2);
        ex0 = zeros(t+50,n);
        ex0(1,:) = vx(1,:);
        for s = 2:(t+50)
            ex0(s,:) = rhox.*ex0(s-1,:)+vx(s,:);
        end
        g_x1 = (0.25+0.5*rand(n,1));
        g_x2 = (0.5*rand(n,1));
        x = f(1,:)'*g_x1'+f(2,:)'*g_x2'+ex0(51:t+50,:);
        
        % factor loading
        l_y1  = g_y1.*rootsig2_y';

        switch errortype
            case 1
                xi = randn(n,t);
            case 2
                xi = (chi2rnd(2,n,t)-2)/2;
            case 3
            %    xi = trnd(4,n,t)/sqrt(2);   %3,4,6,8,12
		xi = randn(n,t);
                xi = sqrt((6-2)./chi2rnd(6,1,t)).*xi;
        end
        W0 = zeros(n,n);
        W0(1,1:3) = 1;
        W0(2,1:4) = 1;
        W0(n-1,n-3:n) = 1;
        W0(n,n-2:n)   = 1;
        for i = 3:n-2
            W0(i,i-2:i+2) = 1;
        end
        % standardize the weight matrix
        W = W0./sum(W0,2);
        Winv = pinv(eye(n)-rho*W);% derive the inverse of IN-rho*W
        c    = sqrt(n/trace(Winv*Winv'));
        e_y  = c*Winv*xi;   % n x t
        e_y  = e_y';   %t x n
        
            
        % Generate data
        y       = zeros(t,n);
        y0      = zeros(t,n);
        a       = 1+sqrt(2)*randn(n,1);
        beta1 = zeros(n,1);
        beta2 = zeros(n,1);
        for i = 1:n
            beta1(i)    = 0.5+0.5*randn(1,1);%1+randn(1,1)
            beta2(i)    = 0.5+0.5*randn(1,1);%1+randn(1,1)
            y0(:,i)  = f(1,:)'*l_y1(i) + rootsig2_y(i)*e_y(:,i);
            y(:,i)   = a(i)+(x(:,i)*beta1(i)+g'*beta2(i))*rootsig2_y(i) + y0(:,i);   
        end
        
        % compute the residuals for CD            
        betax  = zeros(4,n);
        res0     = zeros(t,n);
        for i = 1:n
            xn = [ones(t,1),x(:,i),g',f(1,:)'];
            betax(:,i) = pinv(xn'*xn)*xn'*y(:,i);
            res0(:,i)    = y(:,i)-xn*betax(:,i);
        end
        
        PR0(itr)    = 1-res0(:)'*res0(:)/sum(sum((y-mean(y)).^2)); 
        
        % compute CD0 by assuming f is known            
        rhohats0   = corrcoef(res0); % N x N           
        allrho0    = triu(rhohats0,1); 
        res_cov0   = t^(-1)*(res0'*res0); % N x N
        res_var0   = diag(res_cov0); % N x 1              

        CDscaling   = sqrt(2*t/(n*(n-1)));
        CDtemp0     = CDscaling*sum(allrho0(:));
        CD0(itr)    = CDtemp0;
        
        
        
        
        % compute the residuals for CD
        xb     = mean(x,2);
        yb     = mean(y,2);
        betax  = zeros(5,n);
        yx     = zeros(t,n);     
        xf = [yb,xb,g',ones(t,1)];
        Mn = eye(t)-xf*pinv(xf'*xf)*xf';

        for i = 1:n
            xn = [ones(t,1),g',x(:,i)];
            betax(3,i) = pinv(xn(:,3)'*Mn*xn(:,3))*xn(:,3)'*Mn*y(:,i);
            betax(1:2,i) = pinv(xn(:,1:2)'*xn(:,1:2))*xn(:,1:2)'*(y(:,i)-xn(:,3)*betax(3,i));
            yx(:,i)    = y(:,i)-xn(:,1:3)*betax(1:3,i);
        end
      
        
        % standardize and PCA
        ys      = (yx-mean(yx))./std(yx);
        [PC1,PC2] = eig(ys*ys');              
        
        % m=1
        ft       = PC1(:,end);
        betax   = zeros(4,n);
        for i = 1:n
            xn = [ones(t,1),x(:,i),g',ft];
            betax(:,i) = pinv(xn'*xn)*xn'*y(:,i);
            res0(:,i)    = y(:,i)-xn*betax(:,i);
        end


        PR1(itr)     = 1-res0(:)'*res0(:)/sum(sum((y-mean(y)).^2)); 
        
        xx      = [ones(t,1),ft];
        beta    = pinv(xx'*xx)*xx'*ys;
        res1       = ys-xx*beta;
        betasum = diag(sqrt(beta(2:end,:)*beta(2:end,:)'/n));
        gmma1    = pinv(diag(betasum,0))*beta(2:end,:);
        
        out1 = CD_generation(res1,n,t,gmma1);
        CD1(itr) = out1.CD;
        CDs1(itr) = out1.CDs;
        CDw1(itr) = out1.CDw;
        CDwn1(itr) = out1.CDwn;
        CDzn1(itr) = out1.CDzn;
        
        % m=2
        ft       = PC1(:,end-1:end);
        betax   = zeros(5,n);
        for i = 1:n
            xn = [ones(t,1),x(:,i),g',ft];
            betax(:,i) = pinv(xn'*xn)*xn'*y(:,i);
            res0(:,i)    = y(:,i)-xn*betax(:,i);
        end
        PR2(itr)     = 1-res0(:)'*res0(:)/sum(sum((y-mean(y)).^2)); 
        
        xx      = [ones(t,1),ft];
        beta    = pinv(xx'*xx)*xx'*ys;
        res2       = ys-xx*beta;
        betasum = diag(sqrt(beta(2:end,:)*beta(2:end,:)'/n));
        gmma2    = pinv(diag(betasum,0))*beta(2:end,:);
        
        out2 = CD_generation(res2,n,t,gmma2);
        CD2(itr) = out2.CD;
        CDs2(itr) = out2.CDs;
        CDw2(itr) = out2.CDw;
        CDwn2(itr) = out2.CDwn;
        CDzn2(itr) = out2.CDzn;       
    end
    allCD0    = cat(2,allCD0, CD0);
    allPR0    = cat(2,allPR0,PR0);
    allCD1     = cat(2,allCD1,CD1);
    allCDs1    = cat(2,allCDs1,CDs1);
    allCDw1    = cat(2,allCDw1,CDw1);
    allCDwn1   = cat(2,allCDwn1,CDwn1);
    allCDzn1   = cat(2,allCDzn1,CDzn1);
    
    allPR1     =  cat(2,allPR1,PR1); 
    
    allCD2     = cat(2,allCD2,CD2);
    allCDs2    = cat(2,allCDs2,CDs2);
    allCDw2    = cat(2,allCDw2,CDw2);
    allCDwn2   = cat(2,allCDwn2,CDwn2);
    allCDzn2   = cat(2,allCDzn2,CDzn2);
    
    allPR2     = cat(2,allPR2,PR2); 
    end
end
means    = [mean(allCD0,1);mean(allCD1,1);mean(allCDs1,1);mean(allCDw1,1);...
    mean(allCDwn1,1);mean(allCDzn1,1);...
    mean(allCD2,1);mean(allCDs2,1);mean(allCDw2,1);
    mean(allCDwn2,1);mean(allCDzn2,1)
    ];
stds     = [std(allCD0,1);std(allCD1,1);std(allCDs1,1);std(allCDw1,1);...
    std(allCDwn1,1);std(allCDzn1,1);...
    std(allCD2,1);std(allCDs2,1);std(allCDw2,1);...
    std(allCDwn2,1);std(allCDzn2,1)];
rejrates = [mean(abs(allCD0) > 1.96);mean(abs(allCD1) > 1.96);mean(abs(allCDs1) > 1.96);...
    mean(abs(allCDw1) > 1.96);mean(abs(allCDwn1) > 1.96);mean(abs(allCDzn1) > 1.96);...
    mean(abs(allCD2) > 1.96);mean(abs(allCDs2) > 1.96);...
    mean(abs(allCDw2) > 1.96);mean(abs(allCDwn2) > 1.96);mean(abs(allCDzn2) > 1.96);...
    ];
rejrates = round(100*rejrates,1);
PR       = [mean(allPR0);mean(allPR1);mean(allPR2)];

% arrange the data to the required table form
means    = reshaper(means,55,5);
stds     = reshaper(stds,55,5);
rejrates = reshaper(rejrates,55,5);

PR       = reshaper(PR,15,5);

end
function a=reshaper(A,a,b)
%reshape
a=reshape(A.',b,a).';
end
